﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Operators;
using HeuristicLab.Common;
using HeuristicLab.Parameters;

namespace HeuristicLab.Algorithms.CellularGeneticAlgorithm {
  [Item("VonNeumannNeighborhoodSelector", "Selects a neighborhood for the cellular GA.")]
  [StorableClass]
  public class VonNeumannNeighborhoodSelector: NeighborhoodSelector {
    [StorableConstructor]
    private VonNeumannNeighborhoodSelector(bool deserializing) : base(deserializing) { }

    public VonNeumannNeighborhoodSelector()
      : base() {
    }

    private VonNeumannNeighborhoodSelector(VonNeumannNeighborhoodSelector original, Cloner cloner)
      : base(original, cloner) {
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new VonNeumannNeighborhoodSelector(this, cloner);
    }

    private IScope Get(List<IScope> population, int x, int y) {
      int rows = (int)Math.Ceiling(Math.Sqrt(population.Count));
      int index = (y * rows + x) % population.Count;
      return population[index];
    }

    protected override List<IScope> Select(IScope individual, List<IScope> population) {
      List<IScope> result = new List<IScope>();

      int rows = (int)Math.Ceiling(Math.Sqrt(population.Count));
      int index = population.IndexOf(individual);
      int x = index % rows;
      int y = index / rows;

      result.Add(Get(population, x, y - 1 < 0 ? rows - 1 : y - 1));
      result.Add(Get(population, x, (y + 1) % rows));
      result.Add(Get(population, (x + 1) % rows, y));
      result.Add(Get(population, x - 1 < 0 ? rows - 1 : x - 1, y));
      
      return result;
    }
  }
}
